@@ -62,7 +62,7 @@ static void movecontrolfiles(const char *thing) {
   if (pid == 0) {
     command_shell(buf, _("shell command to move files"));
   }
-  subproc_wait_check(pid, _("shell command to move files"), 0);
+  subproc_reap(pid, _("shell command to move files"), 0);
 }
 
 static void DPKG_ATTR_NORET
@@ -130,7 +130,7 @@ extracthalf(const char *debar, const char *dir,
   if (fstat(arfd, &stab))
     ohshite(_("failed to fstat archive"));
 
-  r = read_line(arfd, versionbuf, strlen(DPKG_AR_MAGIC), sizeof(versionbuf));
+  r = read_line(arfd, versionbuf, strlen(DPKG_AR_MAGIC), sizeof(versionbuf) - 1);
   if (r < 0)
     read_fail(r, debar, _("archive magic version number"));
 
@@ -245,7 +245,7 @@ extracthalf(const char *debar, const char *dir,
     if (errstr)
       ohshit(_("archive has invalid format version: %s"), errstr);
 
-    r = read_line(arfd, ctrllenbuf, 1, sizeof(ctrllenbuf));
+    r = read_line(arfd, ctrllenbuf, 1, sizeof(ctrllenbuf) - 1);
     if (r < 0)
       read_fail(r, debar, _("archive control member size"));
     if (sscanf(ctrllenbuf, "%jd%c%d", &ctrllennum, &nlc, &dummy) != 2 ||
@@ -356,12 +356,12 @@ extracthalf(const char *debar, const char *dir,
       command_exec(&cmd);
     }
     close(p2[0]);
-    subproc_wait_check(c3, "tar", 0);
+    subproc_reap(c3, "tar", 0);
   }
 
-  subproc_wait_check(c2, _("<decompress>"), PROCPIPE);
+  subproc_reap(c2, _("<decompress>"), SUBPROC_NOPIPE);
   if (c1 != -1)
-    subproc_wait_check(c1, _("paste"), 0);
+    subproc_reap(c1, _("paste"), 0);
   if (version.major == 0 && admininfo) {
     /* Handle the version as a float to preserve the behaviour of old code,
      * because even if the format is defined to be padded by 0's that might
@@ -376,6 +376,23 @@ extracthalf(const char *debar, const char *dir,
   }
 }
 
+int
+do_ctrltarfile(const char *const *argv)
+{
+  const char *debar;
+
+  debar = *argv++;
+  if (debar == NULL)
+    badusage(_("--%s needs a .deb filename argument"), cipaction->olong);
+  if (*argv)
+    badusage(_("--%s takes only one argument (.deb filename)"),
+             cipaction->olong);
+
+  extracthalf(debar, NULL, DPKG_TAR_PASSTHROUGH, 1);
+
+  return 0;
+}
+
 int
 do_fsystarfile(const char *const *argv)
 {
