@@ -88,13 +88,21 @@ struct perpackagestate {
 
 class packagelist : public baselist {
 protected:
-  int status_width, section_width, priority_width;
-  int package_width, versioninstalled_width, versionavailable_width, description_width;
-  int section_column, priority_column, versioninstalled_column;
-  int versionavailable_column, package_column, description_column;
+  column col_status;
+  column col_section;
+  column col_priority;
+  column col_package;
+  column col_archinstalled;
+  column col_archavailable;
+  column col_versioninstalled;
+  column col_versionavailable;
+  column col_description;
 
   // Only used when ‘verbose’ is set
-  int status_hold_width, status_status_width, status_want_width;
+  column col_status_hold;
+  column col_status_status;
+  column col_status_old_want;
+  column col_status_new_want;
 
   // Table of packages
   struct perpackagestate *datatable;
@@ -105,6 +113,7 @@ protected:
   bool recursive, verbose;
   enum { so_unsorted, so_section, so_priority, so_alpha } sortorder;
   enum { sso_unsorted, sso_avail, sso_state } statsortorder;
+  enum { ado_none, ado_available, ado_both } archdisplayopt;
   enum { vdo_none, vdo_available, vdo_both } versiondisplayopt;
   bool calcssadone, calcsssdone;
   struct perpackagestate *headings;
@@ -198,6 +207,7 @@ protected:
   void kd_info();
   void kd_toggleinfo();
   void kd_verbose();
+  void kd_archdisplay();
   void kd_versiondisplay();
 
   packagelist(keybindings *kb); // nonrecursive
