@@ -83,8 +83,8 @@ while (@ARGV) {
 	$options{changelogformat} = $1;
     } elsif (m/^-l(.+)$/) {
 	$options{file} = $1;
-    } elsif (m/^-S(.+)$/) {
-	$fieldname = $1;
+    } elsif (m/^-S(.+)?$/) {
+	$fieldname = $1 // shift;
     } elsif (m/^--show-field(?:=(.+))?$/) {
 	$fieldname = $1 // shift(@ARGV);
     } elsif (m/^--$/) {
@@ -105,12 +105,8 @@ while (@ARGV) {
 	    $options{until} = $2;
 	    ## use critic
 	}
-    } elsif (m/^--(count|file|format|from|offset|since|to|until)(.*)$/) {
-	if ($2) {
-	    $options{$1} = $2;
-	} else {
-	    $options{$1} = shift(@ARGV);
-	}
+    } elsif (m/^--(count|file|format|from|offset|since|to|until)(?:=(.+))?$/) {
+        $options{$1} = $2 // shift;
     } elsif (m/^--all$/) {
 	$options{all} = undef;
     } elsif (m/^-(?:\?|-help)$/) {
